extern int sys_exit __P((int, int, int));
extern int sys_vfork  __P((int, int, int));
extern int sys_read __P((int, int, int));
extern int sys_write __P((int, int, int));
extern int sys_open __P((int, int, int));
extern int sys_close __P((int, int, int));
extern int sys_wait  __P((int, int, int));
extern int sys_creat __P((int, int, int));
extern int sys_link  __P((int, int, int));
extern int sys_unlink __P((int, int, int));
extern int sys_waitpid  __P((int, int, int));
extern int sys_chdir __P((int, int, int));
extern int sys_time  __P((int, int, int));
extern int sys_mknod  __P((int, int, int));
extern int sys_chmod  __P((int, int, int));
extern int sys_chown  __P((int, int, int));
extern int sys_brk  __P((int, int, int));
extern int sys_stat  __P((int, int, int));
extern int sys_lseek  __P((int, int, int));
extern int sys_getpid  __P((int, int, int));
extern int sys_mount  __P((int, int, int));
extern int sys_umount  __P((int, int, int));
extern int sys_setuid  __P((int, int, int));
extern int sys_getuid  __P((int, int, int));
extern int sys_stime  __P((int, int, int));
extern int sys_ptrace  __P((int, int, int));
extern int sys_alarm  __P((int, int, int));
extern int sys_fstat  __P((int, int, int));
extern int sys_pause  __P((int, int, int));
extern int sys_utime  __P((int, int, int));
extern int sys_stty  __P((int, int, int));
extern int sys_gtty  __P((int, int, int));
extern int sys_access  __P((int, int, int));
extern int sys_nice  __P((int, int, int));
extern int sys_ftime  __P((int, int, int));
extern int sys_sync  __P((int, int, int));
extern int sys_kill  __P((int, int, int));
extern int sys_rename  __P((int, int, int));
extern int sys_mkdir  __P((int, int, int));
extern int sys_rmdir  __P((int, int, int));
extern int sys_pipe  __P((int, int, int));
extern int sys_times  __P((int, int, int));
extern int sys_prof  __P((int, int, int));
extern int sys_setgid  __P((int, int, int));
extern int sys_getgid  __P((int, int, int));
extern int sys_signal  __P((int, int, int));
extern int sys_acct  __P((int, int, int));
extern int sys_plock  __P((int, int, int));
extern int sys_ioctl  __P((int, int, int));
extern int sys_fcntl  __P((int, int, int));
extern int sys_exec  __P((int, int, int));
extern int sys_umask  __P((int, int, int));
extern int sys_chroot  __P((int, int, int));
extern int sys_sigaction  __P((int, int, int));
extern int sys_sigsuspend  __P((int, int, int));
extern int sys_sigpending  __P((int, int, int));
extern int sys_sigprocmask  __P((int, int, int));
extern int sys_sigreturn  __P((int, int, int));
extern int sys_reboot  __P((int, int, int));
