	XCHG	AX,BL		; illeg
	XCHG	AX,BYTE [BX]	; illeg
	XCHG	AX,DS		; illeg
	XCHG	AX,#1		; illeg

	XCHG	AX,AX
	XCHG	AX,BX
	XCHG	AX,CX
	XCHG	AX,DX
	XCHG	AX,SP
	XCHG	AX,BP
	XCHG	AX,SI
	XCHG	AX,DI

	XCHG	AX,AX
	XCHG	BX,AX
	XCHG	CX,AX
	XCHG	DX,AX
	XCHG	SP,AX
	XCHG	BP,AX
	XCHG	SI,AX
	XCHG	DI,AX

	XCHG	EAX,EAX
	XCHG	EAX,EBX
	XCHG	EAX,ECX
	XCHG	EAX,EDX
	XCHG	EAX,ESP
	XCHG	EAX,EBP
	XCHG	EAX,ESI
	XCHG	EAX,EDI

	XCHG	EAX,EAX
	XCHG	EBX,EAX
	XCHG	ECX,EAX
	XCHG	EDX,EAX
	XCHG	ESP,EAX
	XCHG	EBP,EAX
	XCHG	ESI,EAX
	XCHG	EDI,EAX

	XCHG	AL,AL
	XCHG	AL,AH
	XCHG	AL,BL
	XCHG	AL,BH
	XCHG	BL,CL
	XCHG	BL,CH
	XCHG	BL,DL
	XCHG	BL,DH

	XCHG	[BX],AL
	XCHG	[BX],AH
	XCHG	[BX],BL
	XCHG	[BX],BH
	XCHG	[BX],CL
	XCHG	[BX],CH
	XCHG	[BX],DL
	XCHG	[BX],DH

	XCHG	AL,[BX]
	XCHG	AH,[BX]
	XCHG	BL,[BX]
	XCHG	BH,[BX]
	XCHG	CL,[BX]
	XCHG	CH,[BX]
	XCHG	DL,[BX]
	XCHG	DH,[BX]

	XCHG	[BX],AX
	XCHG	[BX],BX
	XCHG	[BX],CX
	XCHG	[BX],DX
	XCHG	[BX],SP
	XCHG	[BX],BP
	XCHG	[BX],SI
	XCHG	[BX],DI

	XCHG	AX,[BX]
	XCHG	BX,[BX]
	XCHG	CX,[BX]
	XCHG	DX,[BX]
	XCHG	SP,[BX]
	XCHG	BP,[BX]
	XCHG	SI,[BX]
	XCHG	DI,[BX]

	XCHG	[BX],EAX
	XCHG	[BX],EBX
	XCHG	[BX],ECX
	XCHG	[BX],EDX
	XCHG	[BX],ESP
	XCHG	[BX],EBP
	XCHG	[BX],ESI
	XCHG	[BX],EDI

	XCHG	AX,[EBX]
	XCHG	BX,[EBX]
	XCHG	CX,[EBX]
	XCHG	DX,[EBX]
	XCHG	SP,[EBX]
	XCHG	BP,[EBX]
	XCHG	SI,[EBX]
	XCHG	DI,[EBX]
