/* Copyright (C) 1996 Robert de Bath <rdebath@cix.compulink.co.uk>
 * This file is part of the Linux-8086 C library and is distributed
 * under the GNU Library General Public License.
 */
/* We don't care, ignore GCC's __need hackery */

#ifndef __STDDEF_H
#define __STDDEF_H

#include <sys/types.h>

#ifndef NULL
#define NULL ((void*)0)
#endif

#endif /* __STDDEF_H */
