/* condcode.h - condition codes for bcc */

/* Copyright (C) 1992 Bruce Evans */

#define EQ 0
#define NE 1
#define RA 2
#define RN 3
#define LT 4
#define GE 5
#define LE 6
#define GT 7
#define LO 8
#define HS 9
#define LS 10
#define HI 11
