; INHER.ASM

; INHER opcodes

	AAA
	AAS

	CLC
	CLD
	CLI
	CLTS
	CMC
	CMPSB

	DAA
	DAS

	HLT

	INTO
	INSB

	LAHF
	LEAVE
	LOCK
	LODSB

	MOVSB

	NOP

	OUTSB

	REP
	REPE
	REPNE

	SAHF
	SCASB
	STC
	STD
	STI
	STOSB

	WAIT

; INHER16 and INHER32 opcodes

	USE16

	CBW
	CWD
	CWDE
	CDQ
	CMPSW
	CMPSD

	INSW
	INSD
	IRET
	IRETD

	LODSW
	LODSD

	MOVSW
	MOVSD

	OUTSW
	OUTSD

	POPA
	POPAD
	POPF
	POPFD
	PUSHA
	PUSHAD
	PUSHF
	PUSHFD

	SCASW
	SCASD
	STOSW
	STOSW

	XLAT
	XLATB

	USE32

	CBW
	CWD
	CWDE
	CDQ
	CMPSW
	CMPSD

	INSW
	INSD
	IRET
	IRETD

	LODSW
	LODSD

	MOVSW
	MOVSD

	OUTSW
	OUTSD

	POPA
	POPAD
	POPF
	POPFD
	PUSHA
	PUSHAD
	PUSHF
	PUSHFD

	SCASW
	SCASD
	STOSW
	STOSW

	XLAT
	XLATB
