/* flag.h - global structured-flag variables for assembler */

EXTERN struct flags_s list;	/* listing on/off */
EXTERN struct flags_s maclist;	/* list macros on/off */
EXTERN struct flags_s as_warn;	/* warnings on/off */
