/* paths.h <ndf@linux.mit.edu> */

#ifndef ___PATHS_H
#define	___PATHS_H


#define	_PATH_CONSOLE	"/dev/console"
#define	_PATH_TTY	"/dev/tty"
#define	_PATH_NOLOGIN	"/etc/nologin"
#define	_PATH_LOGIN	"/bin/login"
#define	_PATH_BSHELL	"/bin/sh"
#define _PATH_UTMP	"/var/run/utmp"
#define _PATH_WTMP	"/var/log/wtmp"
#define	_PATH_DEFPATH	"/bin:/usr/bin:/usr/local/bin:."
#define	_PATH_DEV	"/dev/"
#define _PATH_DEVNULL	"/dev/null"
#define	_PATH_TMP	"/tmp/"
#define _PATH_LASTLOG	"/var/log/lastlog"
#define _PATH_LOCALE	"/usr/lib/locale"

#endif /* __PATHS_H */
