	MOV	AX,[BX+SI]
	MOV	AX,[BX+DI]
	MOV	AX,[BP+SI]
	MOV	AX,[BP+DI]
	MOV	AX,[SI]
	MOV	AX,[DI]
	MOV	AX,[0x1234]
	MOV	AX,[BX]

	MOV	AX,[BX+SI+0x12]
	MOV	AX,[BX+DI+0x12]
	MOV	AX,[BP+SI+0x12]
	MOV	AX,[BP+DI+0x12]
	MOV	AX,[SI+0x12]
	MOV	AX,[DI+0x12]
	MOV	AX,[BP+0x12]
	MOV	AX,[BX+0x12]

	MOV	AX,[BX+SI+0x1234]
	MOV	AX,[BX+DI+0x1234]
	MOV	AX,[BP+SI+0x1234]
	MOV	AX,[BP+DI+0x1234]
	MOV	AX,[SI+0x1234]
	MOV	AX,[DI+0x1234]
	MOV	AX,[BP+0x1234]
	MOV	AX,[BX+0x1234]

	MOV	AL,AL
	MOV	AL,AH
	MOV	AL,BL
	MOV	AL,BH
	MOV	AL,CL
	MOV	AL,CH
	MOV	AL,DL
	MOV	AL,DH

	MOV	AX,AX
	MOV	AX,CX
	MOV	AX,DX
	MOV	AX,BX
	MOV	AX,SP
	MOV	AX,BP
	MOV	AX,SI
	MOV	AX,DI

	MOV	AX,[EAX]
	MOV	AX,[ECX]
	MOV	AX,[EDX]
	MOV	AX,[EBX]

	MOV	AX,[0x12345678]
	MOV	AX,[ESI]
	MOV	AX,[EDI]

	MOV	AX,[EAX+0x12]
	MOV	AX,[ECX+0x12]
	MOV	AX,[EDX+0x12]
	MOV	AX,[EBX+0x12]

	MOV	AX,[EBP+0x12]
	MOV	AX,[ESI+0x12]
	MOV	AX,[EDI+0x12]

	MOV	AX,[EAX+0x12345678]
	MOV	AX,[ECX+0x12345678]
	MOV	AX,[EDX+0x12345678]
	MOV	AX,[EBX+0x12345678]

	MOV	AX,[EBP+0x12345678]
	MOV	AX,[ESI+0x12345678]
	MOV	AX,[EDI+0x12345678]

	MOV	EAX,EAX
	MOV	EAX,ECX
	MOV	EAX,EDX
	MOV	EAX,EBX
	MOV	EAX,ESP
	MOV	EAX,EBP
	MOV	EAX,ESI
	MOV	EAX,EDI

	MOV	AX,[EAX+ESI*2]
	MOV	AX,[ECX+ESI*2]
	MOV	AX,[EDX+ESI*2]
	MOV	AX,[EBX+ESI*2]
	MOV	AX,[ESP+ESI*2]
	MOV	AX,[ESI*2+0x12345678]
	MOV	AX,[ESI+ESI*2]
	MOV	AX,[EDI+ESI*2]

	MOV	AX,[EAX+ESI*2+0x12]
	MOV	AX,[ECX+ESI*2+0x12]
	MOV	AX,[EDX+ESI*2+0x12]
	MOV	AX,[EBX+ESI*2+0x12]
	MOV	AX,[ESP+ESI*2+0x12]
	MOV	AX,[ESP+0x12]
	MOV	AX,[EBP+ESI*2+0x12]
	MOV	AX,[ESI+ESI*2+0x12]
	MOV	AX,[EDI+ESI*2+0x12]

	MOV	AX,[EAX+ESI*2+0x12345678]
	MOV	AX,[ECX+ESI*2+0x12345678]
	MOV	AX,[EDX+ESI*2+0x12345678]
	MOV	AX,[EBX+ESI*2+0x12345678]
	MOV	AX,[ESP+ESI*2+0x12345678]
	MOV	AX,[ESP+0x12345678]
	MOV	AX,[EBP+ESI*2+0x12345678]
	MOV	AX,[ESI+ESI*2+0x12345678]
	MOV	AX,[EDI+ESI*2+0x12345678]
