/* const.h - constants for linker */

/* Copyright (C) 1994 Bruce Evans */

#define FALSE	0
#define NUL_PTR	((void*)0)
#define TRUE	1

#define EXTERN	extern
#define FORWARD	static
#define PRIVATE	static
#define PUBLIC

#include "config.h"
