static char *copyright =
   "@(#) Copyright (C) 1987 G. M. Harding, all rights reserved";

#ifdef __linux__
char *release =
   "released with Dev86 for Linux, based on minix 2.1";
#else
static char *sccsid =
   "@(#) disrel.c, Ver. 2.1 created 00:00:00 87/09/01";

char *release =
   "release 2.1 (MINIX)";
#endif

 /*
 **
 ** This file documents the major revisions to the 8088 sym-
 ** bolic disassembler. It also contains the release string
 ** which is output at the head of each disassembly, and the
 ** copyright string which must be incorporated in any code
 ** distribution.
 **
 ** Permission to copy and redistribute is hereby granted,
 ** provided full source code, with all copyright notices,
 ** accompanies any redistribution.
 **
 ** REVISION HISTORY:
 **
 ** SEP 87:
 **    After internal shakeout, released on Usenet.
 **
 ** JUN 88:
 **    Ported to MINIX
 */

