mov eax,cr0
mov eax,cr2
mov eax,cr3
mov eax,dr0
mov eax,dr1
mov eax,dr2
mov eax,dr3
mov eax,dr6
mov eax,dr7
mov eax,tr3
mov eax,tr4
mov eax,tr5
mov eax,tr6
mov eax,tr7

mov cr0,eax
mov cr2,eax
mov cr3,eax
mov dr0,eax
mov dr1,eax
mov dr2,eax
mov dr3,eax
mov dr6,eax
mov dr7,eax
mov tr3,eax
mov tr4,eax
mov tr5,eax
mov tr6,eax
mov tr7,eax


mov ebx,cr0
mov ebx,cr2
mov ebx,cr3
mov ebx,dr0
mov ebx,dr1
mov ebx,dr2
mov ebx,dr3
mov ebx,dr6
mov ebx,dr7
mov ebx,tr3
mov ebx,tr4
mov ebx,tr5
mov ebx,tr6
mov ebx,tr7

mov cr0,ebx
mov cr2,ebx
mov cr3,ebx
mov dr0,ebx
mov dr1,ebx
mov dr2,ebx
mov dr3,ebx
mov dr6,ebx
mov dr7,ebx
mov tr3,ebx
mov tr4,ebx
mov tr5,ebx
mov tr6,ebx
mov tr7,ebx


mov ecx,cr0
mov ecx,cr2
mov ecx,cr3
mov ecx,dr0
mov ecx,dr1
mov ecx,dr2
mov ecx,dr3
mov ecx,dr6
mov ecx,dr7
mov ecx,tr3
mov ecx,tr4
mov ecx,tr5
mov ecx,tr6
mov ecx,tr7

mov cr0,ecx
mov cr2,ecx
mov cr3,ecx
mov dr0,ecx
mov dr1,ecx
mov dr2,ecx
mov dr3,ecx
mov dr6,ecx
mov dr7,ecx
mov tr3,ecx
mov tr4,ecx
mov tr5,ecx
mov tr6,ecx
mov tr7,ecx


mov edx,cr0
mov edx,cr2
mov edx,cr3
mov edx,dr0
mov edx,dr1
mov edx,dr2
mov edx,dr3
mov edx,dr6
mov edx,dr7
mov edx,tr3
mov edx,tr4
mov edx,tr5
mov edx,tr6
mov edx,tr7

mov cr0,edx
mov cr2,edx
mov cr3,edx
mov dr0,edx
mov dr1,edx
mov dr2,edx
mov dr3,edx
mov dr6,edx
mov dr7,edx
mov tr3,edx
mov tr4,edx
mov tr5,edx
mov tr6,edx
mov tr7,edx


mov esi,cr0
mov esi,cr2
mov esi,cr3
mov esi,dr0
mov esi,dr1
mov esi,dr2
mov esi,dr3
mov esi,dr6
mov esi,dr7
mov esi,tr3
mov esi,tr4
mov esi,tr5
mov esi,tr6
mov esi,tr7

mov cr0,esi
mov cr2,esi
mov cr3,esi
mov dr0,esi
mov dr1,esi
mov dr2,esi
mov dr3,esi
mov dr6,esi
mov dr7,esi
mov tr3,esi
mov tr4,esi
mov tr5,esi
mov tr6,esi
mov tr7,esi


mov edi,cr0
mov edi,cr2
mov edi,cr3
mov edi,dr0
mov edi,dr1
mov edi,dr2
mov edi,dr3
mov edi,dr6
mov edi,dr7
mov edi,tr3
mov edi,tr4
mov edi,tr5
mov edi,tr6
mov edi,tr7

mov cr0,edi
mov cr2,edi
mov cr3,edi
mov dr0,edi
mov dr1,edi
mov dr2,edi
mov dr3,edi
mov dr6,edi
mov dr7,edi
mov tr3,edi
mov tr4,edi
mov tr5,edi
mov tr6,edi
mov tr7,edi


mov esp,cr0
mov esp,cr2
mov esp,cr3
mov esp,dr0
mov esp,dr1
mov esp,dr2
mov esp,dr3
mov esp,dr6
mov esp,dr7
mov esp,tr3
mov esp,tr4
mov esp,tr5
mov esp,tr6
mov esp,tr7

mov cr0,esp
mov cr2,esp
mov cr3,esp
mov dr0,esp
mov dr1,esp
mov dr2,esp
mov dr3,esp
mov dr6,esp
mov dr7,esp
mov tr3,esp
mov tr4,esp
mov tr5,esp
mov tr6,esp
mov tr7,esp


mov ebp,cr0
mov ebp,cr2
mov ebp,cr3
mov ebp,dr0
mov ebp,dr1
mov ebp,dr2
mov ebp,dr3
mov ebp,dr6
mov ebp,dr7
mov ebp,tr3
mov ebp,tr4
mov ebp,tr5
mov ebp,tr6
mov ebp,tr7

mov cr0,ebp
mov cr2,ebp
mov cr3,ebp
mov dr0,ebp
mov dr1,ebp
mov dr2,ebp
mov dr3,ebp
mov dr6,ebp
mov dr7,ebp
mov tr3,ebp
mov tr4,ebp
mov tr5,ebp
mov tr6,ebp
mov tr7,ebp
